<?php /*

**************************************************************************

Plugin Name:  bbPress2 shortcode whitelist
Plugin URI:   http://bbpressbbcode.chantech.org/
Description:  Adds a safe whitelist of shortcodes to bbpress
Version:      0.1
Author:       Anton Channing
Author URI:   http://ant.chantech.org

**************************************************************************

Copyright (C) 2011 Anton Channing

***** GPL3 *****
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

**************************************************************************/

class bbPressShortcodeWhitelist {
	public $bbp_shortcode_whitelist = array();

	// Plugin initialization
	function __construct() {
		// This version only supports WP 2.5+ (learn to upgrade please!)
		if ( !function_exists('add_shortcode') ) return;

		// Lets assume bbcode is always safe for now.  In long run we should implement this as a
		// customisable admin setting.  Possibly we could auto detect certain popular plugins that
		// add safe shortcodes (ie Vipers video quicktags) and allow a bulk add of those.
		$this->bbp_shortcode_whitelist = array_merge(array('b','i','u','s','img','url','center','quote','color','code','size','ul','ol','li','youtube','gvideo','spoiler'),$this->bbp_shortcode_whitelist);

	}	

	function add_filters() {
		// Add safe shortcode support to relevant filters 
		add_filter( 'get_comment_text', 'bbp_whitelist_do_shortcode' ); //adds safe shortcodes to blog comments
		//add_filter( 'bbp_get_topic_content', 'bbp_whitelist_do_shortcode' ); //adds safe shortcodes to forum topics
		add_filter( 'bbp_get_reply_content', 'bbp_whitelist_do_shortcode' ); //adds safe shortcodes to forum replies
	}
}

$bbp_sc_whitelist = new bbPressShortcodeWhitelist();

// Start this plugin once all other plugins are fully loaded
add_action( 'plugins_loaded', create_function( '', 'global $bbp_sc_whitelist; $bbp_sc_whitelist->add_filters();' ) );

	// These are modified versions of the wordpress functions for doing shortcodes,
	// that work with a 'safelist' of shortcodes instead.  Probably this functionality
	// should be branched out to a new plugin to make it more universal.
	function bbp_whitelist_do_shortcode($content) {
		global $shortcode_tags;
		global $bbp_sc_whitelist;

		if (empty($shortcode_tags) || !is_array($shortcode_tags))
			return $content;

		$tagnames = array();
		foreach($shortcode_tags as $tag => $func) {
			if(in_array(strtolower($tag),$bbp_sc_whitelist->bbp_shortcode_whitelist)) $tagnames[] = $tag;
		}

		if (empty($tagnames)) return $content;

		$pattern = bbp_whitelist_get_shortcode_regex();
		return preg_replace_callback('/'.$pattern.'/s', 'bbp_whitelist_do_shortcode_tag', $content);
	}
		
	function bbp_whitelist_get_shortcode_regex() {
		global $shortcode_tags;
		global $bbp_sc_whitelist;

		$tagnames = array();
		foreach($shortcode_tags as $tag => $func) {
			if(in_array(strtolower($tag),$bbp_sc_whitelist->bbp_shortcode_whitelist)) $tagnames[] = $tag;
		}

		$tagregexp = join( '|', array_map('preg_quote', $tagnames) );

		// WARNING! Do not change this regex without changing do_shortcode_tag() and strip_shortcodes()
		return '(.?)\[('.$tagregexp.')\b(.*?)(?:(\/))?\](?:(.+?)\[\/\2\])?(.?)';
	}

	function bbp_whitelist_do_shortcode_tag( $m ) {
		global $shortcode_tags;
		global $bbp_sc_whitelist;

		$safe_sortcode_tags = array();
		foreach($shortcode_tags as $tag => $func) {
			if(in_array(strtolower($tag),$bbp_sc_whitelist->bbp_shortcode_whitelist)) $safe_sortcode_tags[$tag] = $func;
		}


		// allow [[foo]] syntax for escaping a tag
		if ( $m[1] == '[' && $m[6] == ']' ) {
			return substr($m[0], 1, -1);
		}

		$tag = $m[2];
		$attr = shortcode_parse_atts( $m[3] );

		if(!isset($safe_sortcode_tags[$tag])) return $m[0]; // Not in safe list, so return tag unchanged.

		if ( isset( $m[5] ) ) {
			// enclosing tag - extra parameter
			return $m[1] . call_user_func( $safe_sortcode_tags[$tag], $attr, $m[5], $tag ) . $m[6];
		} else {
			// self-closing tag
			return $m[1] . call_user_func( $safe_sortcode_tags[$tag], $attr, NULL,  $tag ) . $m[6];
		}
	}
?>
